#include <arduinoFFT.h>
#include <Adafruit_NeoPixel.h> 

#define micPin A0

#define LED_PIN 6 
#define NUM_LEDS 24 

#define red_range 10 
#define green_range 11
#define blue_range 10

#define SAMPLES 64 
#define SAMPLING_FREQUENCY 1000 


unsigned int sampling_period_us;
unsigned long microseconds;

double vReal[SAMPLES];
double vImag[SAMPLES];

ArduinoFFT<double> FFT = ArduinoFFT<double>(vReal, vImag, SAMPLES, SAMPLING_FREQUENCY);

double red_ampl = 0;
double green_ampl = 0;
double blue_ampl = 0;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800); 

void setup() {
  Serial.begin(115200);
  sampling_period_us = round(1000000 * (1.0 / SAMPLING_FREQUENCY));

  setPinAsInput(14);
  setPinAsOutput(LED_PIN);

  strip.begin(); 
}

void loop() {
  for (int i = 0; i < SAMPLES; i++) {
    microseconds = micros();
    vReal[i] = analogRead(micPin);
    vImag[i] = 0;

    while (micros() < (microseconds + sampling_period_us)) {}
  }

  FFT.windowing(vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.compute(vReal, vImag, SAMPLES, FFT_FORWARD);
  FFT.complexToMagnitude(vReal, vImag, SAMPLES);

  red_ampl = 0;
  green_ampl = 0;
  blue_ampl = 0;

  for (int i = 1; i < SAMPLES / 2; i++) {
    if (i <= red_range) red_ampl += vReal[i];
    else if (i >= (red_range + 1) && i <= (red_range + green_range)) green_ampl += vReal[i];
    else blue_ampl += vReal[i];
  }

  red_ampl /= red_range;
  green_ampl /= green_range;
  blue_ampl /= blue_range;

  if ((red_ampl > green_ampl) && (green_ampl > blue_ampl)) {
    red_ampl *= 1.5;
    green_ampl *= 0.5;
    blue_ampl *= 0.5;
  } else if ((blue_ampl > red_ampl) && (blue_ampl > green_ampl)) {
    blue_ampl *= 1.5;
    green_ampl *= 0.2;
    red_ampl *= 0.2;
  } else if ((green_ampl > blue_ampl) && (green_ampl > red_ampl)) {
    green_ampl *= 1.5;
    red_ampl *= 0.2;
    blue_ampl *= 0.2;
  }


  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(255 * magnitudeConverter(red_ampl), 255 * magnitudeConverter(green_ampl), 255 * magnitudeConverter(blue_ampl)));
  }
  strip.show(); 

  delay(25);
}

void setPinAsInput(uint8_t pin) {
  if (pin >= 0 && pin <= 7) {
    DDRD &= ~(1 << pin); 
  } else if (pin >= 8 && pin <= 13) {
    DDRB &= ~(1 << (pin - 8)); 
  } else if (pin >= 14 && pin <= 19) {
    DDRC &= ~(1 << (pin - 14)); 
  }
}

void setPinAsOutput(uint8_t pin) {
  if (pin >= 0 && pin <= 7) {
    DDRD |= (1 << pin); 
  } else if (pin >= 8 && pin <= 13) {
    DDRB |= (1 << (pin - 8)); 
  } else if (pin >= 14 && pin <= 19) {
    DDRC |= (1 << (pin - 14)); 
  }
}

double magnitudeConverter(double brightness) {
  double ans = brightness / 614;
  if (ans < 0.2) ans = 0;
  else if (ans > 1) ans = 1.00;
  return ans;
}












